from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    """
    help = (
        "Add an basic set of auth.Groups for cms usage (cms-base, cms-publisher, cms-superuser, plugin-permissions)  that gives basic CMS Permissions. User --plugin or --cms to add additional perms."
        "")

    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            action='store_const',
            const=True,
            required=False,
            help="""
        Force overwrite of existing group and it's permissions.""")

    def handle(self, *args, **options):
        args = []
        if options.get('force'):
            args = ['--force', ]
        call_command('djangocms_misc_add_cms_group', '--cms-base', *args)
        call_command('djangocms_misc_add_cms_group', '--cms-publish', '--name=cms_publisher', *args)
        call_command('djangocms_misc_add_cms_group', '--cms-superuser', '--name=cms_superuser', *args)
        call_command('djangocms_misc_add_cms_group', '--cms', '--plugin', '--name=cms_plugin_permissions', *args)



from django.contrib import admin
from modeltranslation.admin import TranslationTabularInline, TranslationStackedInline, TranslationAdmin

from .models import TestModel, TestInlineModel
from ...basic.admin import LanguageTabsMixin


class TestInlineModelInline(TranslationStackedInline, admin.StackedInline):
    model = TestInlineModel
    extra = 2


class TestInlineModelInline2(TranslationTabularInline, admin.TabularInline):
    model = TestInlineModel
    extra = 2


class TestModelAdmin(LanguageTabsMixin, TranslationAdmin, admin.ModelAdmin):
    inlines = [TestInlineModelInline, TestInlineModelInline2, ]
    fieldsets = [
        ['', {'fields': ['field0', ]}],
        ['First Set', {'fields': ['field1', 'field2']}],
    ]


admin.site.register(TestModel, TestModelAdmin)



# coding: utf-8
from cms.models import Placeholder  # noqa - needed, circular import otherwise
from cms.plugin_rendering import ContentRenderer  # , RenderedPlaceholder


try:
    # cms 3.5 or 3.7+
    from cms.plugin_rendering import StructureRenderer
except ImportError:
    StructureRenderer = None


# load conf at startup
from .conf import UntranslatedPlaceholderConf  # noqa
# import signals at startup
from .signals import *  # noqa (will forget to update otherwise!)
from .utils import get_untranslated_default_language_if_enabled


def new_renderer__init__(self, request):
    self.__original_init__(request)
    lang = get_untranslated_default_language_if_enabled()
    if lang:
        self.request_language = lang


# monkey patch!
# for normal plugin rendering.
ContentRenderer.__original_init__ = ContentRenderer.__init__
ContentRenderer.__init__ = new_renderer__init__


def new_structure_render_placeholder(self, placeholder, language, page=None):
    language = language or self.request_language
    return self.__original_render_placeholder(placeholder, language, page)


if StructureRenderer:
    # for structure mode
    StructureRenderer.__original_init__ = StructureRenderer.__init__
    StructureRenderer.__init__ = new_renderer__init__
    StructureRenderer.__original_render_placeholder = StructureRenderer.render_placeholder
    StructureRenderer.render_placeholder = new_structure_render_placeholder



from django.http import HttpResponsePermanentRedirect

"""
some view mixins, to use with django-cms
"""


class LanguageChooserEnhancerMixin(object):
    """
    this calls correct get_absolute_urls in cms's language_chooser tags
    """
    def get(self, request, **kwargs):
        self.object = self.get_object()
        if hasattr(self.request, 'toolbar'):
            self.request.toolbar.set_object(self.object)
        return super(LanguageChooserEnhancerMixin, self).get(request, **kwargs)


class AutoSlugMixin(object):
    """
    redirect if the slug is no more correct
    """
    def get(self, request, **kwargs):
        self.object = self.get_object()
        if self.request.path != self.object.get_absolute_url():
            return HttpResponsePermanentRedirect(self.object.get_absolute_url())
        return super(AutoSlugMixin, self).get(request, **kwargs)


class PublishedViewMixin():
    """
    in edit mode, get all, otherwise only published
    """
    def get_queryset(self):
        if self.request.toolbar.edit_mode_active:
            return self.model.objects.all()
        return self.model.objects.published()



from django.utils.translation import gettext_lazy as _
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


@apphook_pool.register
class AppTemplateAppHook(CMSApp):
    name = _("AppTemplate App")
    # menus = [CategoryMenu, ]

    def get_urls(self, page=None, language=None, **kwargs):
        return ["apptemplate.urls", ]



import logging
import os
import shutil

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    - copy plugin_template folder to new location
    - search and replace PLUGINTEMPLATE / Plugintemplate / plugintemplate in all files
    - rename template name, test file name
    """
    help = (
        "Bootstrap a new cms based app, providing app hooks and menus extension templates"
        "")

    def add_arguments(self, parser):
        parser.add_argument('path', help="""
The path to copy the files to.
Will take part after last / as package name
""")
        parser.add_argument('app_name', help="""
Name of the new app. CamelCase, as you would name the model: Article/ News / DoorBell.
""")

    def handle(self, *args, **options):
        # Use a stdout logger
        logger = logging.getLogger(__name__)
        stream = logging.StreamHandler(self.stdout)
        logger.addHandler(stream)
        logger.setLevel(logging.DEBUG)

        # copy
        path = options['path']
        app_name = options['app_name']
        current_path = os.path.dirname(__file__)
        source_folder = os.path.join(current_path, '..', '..', 'app_template')
        if not path.startswith('/'):
            target_folder = os.path.join('.', path)
        else:
            target_folder = path
        if os.path.exists(target_folder):
            logger.log(logging.INFO, 'Aborting, target folder already existing: %s' % target_folder)
            return
        shutil.copytree(source_folder, target_folder)

        # replace
        recursive_replace_template(target_folder, 'AppTemplate', app_name)
        recursive_replace_template(target_folder, 'apptemplate', app_name.lower())

        # rename
        base_templates_folder = os.path.join(target_folder, 'templates', )
        templates_folder = os.path.join(base_templates_folder, app_name.lower())
        os.rename(
            os.path.join(base_templates_folder, 'app'),
            templates_folder
        )
        os.rename(
            os.path.join(templates_folder, 'app_list.html'),
            os.path.join(templates_folder, app_name.lower() + '_list.html'),
        )
        os.rename(
            os.path.join(templates_folder, 'app_detail.html'),
            os.path.join(templates_folder, app_name.lower() + '_detail.html'),
        )
        test_folder = os.path.join(target_folder, 'tests', )
        os.rename(
            os.path.join(test_folder, 'tst_app_basic.py'),
            os.path.join(test_folder, 'test_' + app_name.lower() + "_basic.py"),
        )

        logger.log(logging.INFO, 'Success, copied new app files to:')
        logger.log(logging.INFO,  target_folder)
        logger.log(logging.INFO, '---')
        logger.log(logging.INFO, 'Next steps')
        logger.log(logging.INFO, '- Adapt your models.py')
        logger.log(logging.INFO, '- Adapt your views.py (PublishedViewMixin, etc!)')
        logger.log(logging.INFO, '- add your app to INSTALLED_APPS')
        logger.log(logging.INFO, '- run ./manage.py makemigrations')
        logger.log(logging.INFO, '- run ./manage.py test (basic concepts like class names ')
        logger.log(logging.INFO, '  applied, admin form working, adapt to your needs)')


def recursive_replace_template(target_folder, target, replacement):
    for dname, dirs, files in os.walk(target_folder):
        for fname in files:
            if fname[-4:] == '.pyc':
                # nope, dont need em
                continue
            fpath = os.path.join(dname, fname)
            with open(fpath) as f:
                s = f.read()
            s = s.replace(target, replacement)
            with open(fpath, "w") as f:
                f.write(s)



import logging

from django.contrib.auth.models import Group, Permission
from django.core.management.base import BaseCommand
from django.db.models import Q


class Command(BaseCommand):
    """
    """
    help = (
        "Add an auth.Group that gives basic CMS Permissions. User --plugin or --cms to add additional perms."
        "")

    def add_arguments(self, parser):
        parser.add_argument(
            '--cms-base',
            required=False,
            action='store_const',
            const=True,
            default=False,
            help="Basic CMS permissions (edit and publish pages, etc).")
        parser.add_argument(
            '--cms-publish',
            required=False,
            action='store_const',
            const=True,
            default=False,
            help="adds exactly 'can publish page' permission).",
        )
        parser.add_argument(
            '--cms-superuser',
            required=False,
            action='store_const',
            const=True,
            default=False,
            help="Superuser CMS permissions (add own users, page permissions, etc, etc).",
        )
        parser.add_argument(
            '--plugin',
            required=False,
            action='store_const',
            const=True,
            default=False,
            help="""
Searches for permissions with 'plugin' in their content type's app_label (ex.: cmsplugin_googlemaps), adds to group.""")
        parser.add_argument(
            '--cms',
            action='store_const',
            const=True,
            default=False,
            required=False,
            help="""
Searches for permissions with 'cms' in their content type's app_label (ex.:djangocms_text_ckeditor), adds to group.""")
        parser.add_argument(
            '--name',
            required=False,
            default='cms_base',
            help="""
Name of group. Defaults to cms_base.""")
        parser.add_argument(
            '--force',
            action='store_const',
            const=True,
            required=False,
            help="""
Force overwrite of existing group and it's permissions.""")

    def handle(self, *args, **options):
        group_name = options.get('name')
        # copy
        groups = Group.objects.filter(name=group_name)
        if groups.count():
            if not options.get('force', None):
                exit("use --force to overwrite existing groups!")
            if groups.count() > 1:
                exit("more than one group with name that name ({}) found! exiting.".format(group_name))
            group = groups.first()
        else:
            group = Group.objects.create(name=group_name)

        group.permissions.clear()
        if options.get('cms_base'):
            q_filter = Q(content_type__app_label='cms')
            q_exclude = Q(name__icontains='user')
            q_exclude |= Q(name__icontains='permission')
            q_exclude |= Q(name__icontains='publish')
            self.add_permission(group, q_filter, q_exclude)
        if options.get('cms_publish'):
            q_filter = Q(content_type__app_label='cms')
            q_filter &= Q(name__icontains='publish')
            self.add_permission(group, q_filter)
        if options.get('cms_superuser'):
            q_filter = Q(content_type__app_label='cms')
            self.add_permission(group, q_filter)
        if options.get('cms'):
            q_filter = Q(content_type__app_label__icontains='cms')
            q_exclude = Q(content_type__app_label='cms')
            self.add_permission(group, q_filter, q_exclude)
        if options.get('plugin'):
            q_filter = Q(content_type__app_label__icontains='plugin')
            q_exclude = Q(content_type__app_label='cms')
            self.add_permission(group, q_filter, q_exclude)

        print(logging.INFO, 'Group created! ({})'.format(group_name))
        print(logging.INFO, 'Following permissions were granted:')
        for perm in group.permissions.all():
            print(logging.INFO, '{}'.format(perm))

    def add_permission(self, group, q_filter=Q(), q_exclude=Q()):
        perms = Permission.objects.filter(q_filter).exclude(q_exclude)
        group.permissions.add(*perms)



from cms.models import CMSPlugin
from django.db import models


class TestPluginModel(CMSPlugin):
    field1 = models.CharField(max_length=64, default='', blank=False)
    field_date = models.DateField(default=None, null=True, )
    field_datetime = models.DateTimeField(default=None, null=True, )
    field_time = models.TimeField(default=None, null=True, )

    def __str__(self):
        return self.field1


class TestModel(models.Model):
    field0 = models.CharField(max_length=64, default='', blank=True)
    field1 = models.CharField(max_length=64, default='', blank=False)
    field2 = models.CharField(max_length=64, default='', blank=True)

    def __str__(self):
        return self.field1


class TestInlineModel(models.Model):
    testmodel = models.ForeignKey(TestModel, on_delete=models.CASCADE)
    field1 = models.CharField(max_length=64, default='', blank=False)
    field2 = models.CharField(max_length=64, default='', blank=True)

    def __str__(self):
        return self.field1



from cms.menu_bases import CMSAttachMenu  # , Menu
from django.conf import settings
from menus.base import NavigationNode
from django.utils.translation import gettext_lazy as _

from .models import AppTemplate


class AppTemplateMenu(CMSAttachMenu):
    name = _('AppTemplate Menu')

    def get_nodes(self, request):
        # better: check if current Site has a BLOG app somewhere...
        nodes = []
        if settings.SITE_ID == 2:
            categories = AppTemplate.objects.published()
            for cat in categories:
                node = NavigationNode(cat.title, cat.get_absolute_url(), cat.id)
                nodes.append(node)
        return nodes

# menu_pool.register_menu(ArticleMenu)



from cms.admin.placeholderadmin import PlaceholderAdminMixin
from django.contrib import admin

from .models import AppTemplate


@admin.register(AppTemplate)
class AppTemplateAdmin(PlaceholderAdminMixin, admin.ModelAdmin):
    list_display = ('date', 'title', 'published', )
    fieldsets = (
        ('', {
            'fields': (
                'published',
                (
                    'title',
                    'seo_title',
                    'date'
                ),
                'meta_description',
                # 'preview_text',
                'preview_image',
            )
        }),
    )
